from rich import print
from typing import Any, override
from pathlib import Path
from marko.inline import Link, Image
from marko.html_renderer import HTMLRenderer
from marko.parser import Parser

from zona.models import BuildCtx
from zona import util


class ZonaRenderer(HTMLRenderer):
    def __init__(
        self,
        resolve: bool = False,
        source: Path | None = None,
        ctx: BuildCtx | None = None,
    ):
        super().__init__()
        self.resolve: bool = resolve
        if self.resolve:
            # print("Resolve is set")
            assert source is not None
            assert ctx is not None
            self.source: Path = source.resolve()
            self.ctx: BuildCtx = ctx

    @override
    def render_link(self, element: Link):
        href = element.dest
        print(f"Rendering link, original: {href}")
        assert isinstance(href, str)
        if self.resolve:
            cur = Path(href)
            _href = href
            if href.startswith("/"):
                # resolve relative to content root
                resolved = (self.ctx.base / cur.relative_to("/")).resolve()
            else:
                # treat as relative link and try to resolve
                resolved = (self.source.parent / cur).resolve()
            # only substitute if link points to an actual file
            if resolved.exists():
                item = self.ctx.item_map.get(resolved)
                if item:
                    href = util.normalize_url(item.url)
                    print(
                        f"Link in file {self.source}: {_href} resolved to {href}"
                    )
                else:
                    print(
                        f"Warning: resolved path {resolved} not found in item map"
                    )
        body: Any = self.render_children(element)
        return f'<a href="{href}" target="_blank">{body}</a>'

    # TODO: image compression/dithering?
    @override
    def render_image(self, element: Image):
        # get label text from children
        text = self.render_children(element)
        title = element.title or ""
        caption = f"<small>{text}</small>" if text else ""
        return (
            f'<div class="image-container">\n'
            # TODO: convert to plaintext and add as alt attribute
            f'<img src="{element.dest}" title="{title}">\n'
            f"{caption}</div>"
        )


def md_to_html(
    content: str,
    resolve_links: bool = False,
    source: Path | None = None,
    ctx: BuildCtx | None = None,
) -> str:
    if resolve_links and (source is None or ctx is None):
        raise TypeError(
            "md_to_html() missing source and ctx when resolve_links is true"
        )
    parser = Parser()
    ast = parser.parse(content)
    renderer = ZonaRenderer(resolve_links, source, ctx)
    return renderer.render(ast)
