from pathlib import Path
from jinja2 import Environment, FileSystemLoader, select_autoescape
from zona.models import Item
from zona.markdown import md_to_html


def get_header(template_dir: Path) -> str | None:
    md_header = template_dir / "header.md"
    html_header = template_dir / "header.html"
    if md_header.exists():
        return md_to_html(md_header.read_text())
    elif html_header.exists():
        return html_header.read_text()


def get_footer(template_dir: Path) -> str | None:
    md_footer = template_dir / "footer.md"
    html_footer = template_dir / "footer.html"
    if md_footer.exists():
        return md_to_html(md_footer.read_text())
    elif html_footer.exists():
        return html_footer.read_text()


class Templater:
    def __init__(self, template_dir: Path):
        self.env: Environment = Environment(
            loader=FileSystemLoader(template_dir),
            autoescape=select_autoescape(["html", "xml"]),
        )
        self.template_dir: Path = template_dir
        self.header: str | None = get_header(template_dir)
        self.footer: str | None = get_footer(template_dir)

    def render_item(self, item: Item, content: str) -> str:
        env = self.env
        meta = item.metadata
        assert meta is not None
        template = env.get_template(meta.template)
        header = self.header if self.header and meta.header else False
        footer = self.footer if self.footer and meta.footer else False
        return template.render(
            content=content,
            url=item.url,
            metadata=meta,
            header=header,
            footer=footer,
        )
